@@ -2,7 +2,9 @@
  * dEploid is used for deconvoluting Plasmodium falciparum genome from
  * mix-infected patient sample.
  *
- * Copyright (C) 2016, Sha (Joe) Zhu, Jacob Almagro and Prof. Gil McVean
+ * Copyright (C) 2016-2017 University of Oxford
+ *
+ * Author: Sha (Joe) Zhu
  *
  * This file is part of dEploid.
  *
@@ -26,7 +28,7 @@
 #include <iomanip>      // std::setw
 #include <ctime>
 #include <iterator>
-
+#include "updateHap.hpp"
 
 DEploidIO::DEploidIO(){
     this->init();
@@ -61,6 +63,9 @@ DEploidIO::~DEploidIO(){
         delete this->vcfReaderPtr_;
     }
 
+    if ( this->panel != NULL ){
+        delete panel;
+    }
 }
 
 
@@ -86,23 +91,31 @@ void DEploidIO::init() {
     this->setDoExportRecombProb(false);
     this->setRandomSeedWasSet(false);
     this->setCompressVcf(false);
-    this->initialPropWasGiven_ = false;
+    this->setInitialPropWasGiven(false);
+    this->setInitialHapWasGiven(false);
     this->initialProp.clear();
     this->setExcludeSites( false );
     this->excludedMarkers = NULL;
+    this->panel = NULL;
     this->set_seed( (unsigned)0 );
     this->set_help(false);
     this->setVersion(false);
-    this->set_panel(true);
+    this->setUsePanel(true);
     this->precision_ = 8;
     this->prefix_ = "pf3k-dEploid";
-    this->kStrain_ = 5;
+    this->setKStrainWasManuallySet(false);
+    this->setKStrainWasSetByHap(false);
+    this->setKStrainWasSetByProp(false);
+    this->setKstrain(5);
     this->nMcmcSample_ = 800;
-    this->setDoUpdateProp ( true );
-    this->setDoUpdatePair ( true );
-    this->setDoUpdateSingle ( true );
+    this->setDoUpdateProp( true );
+    this->setDoUpdatePair( true );
+    this->setDoUpdateSingle( true );
     this->setDoExportPostProb( false );
-    this->setDoExportSwitchMissCopy ( false );
+    this->setDoPainting( false );
+    this->setUseIBD( false );
+    this->setDoExportSwitchMissCopy ( true );
+    this->setDoAllowInbreeding( false );
     this->mcmcBurn_ = 0.5;
     this->mcmcMachineryRate_ = 5;
     this->missCopyProb_ = 0.01;
@@ -110,7 +123,9 @@ void DEploidIO::init() {
     this->setForbidCopyFromSame( false );
     this->constRecombProb_ = 1.0;
     this->averageCentimorganDistance_ = 15000.0;
-    this->Ne_ = 10.0;
+    this->setScalingFactor(100.0);
+    this->setParameterG(20.0);
+    this->setParameterSigma(5.0);
 
     this->setUseVcf(false);
     this->vcfReaderPtr_ = NULL;
@@ -154,6 +169,10 @@ void DEploidIO::reInit() {
 
 
 void DEploidIO::finalize(){
+    if ( this->useIBD() && this->kStrain() == 1){
+        throw InvalidK();
+    }
+
     if ( this->compressVcf() && !this->doExportVcf() ){
         throw VcfOutUnSpecified("");
     }
@@ -191,6 +210,7 @@ void DEploidIO::finalize(){
         }
         this->altCount_ = alt.info_;
     }
+
     TxtReader plaf;
     plaf.readFromFile(plafFileName_.c_str());
     if ( this->excludeSites() ){
@@ -210,53 +230,79 @@ void DEploidIO::finalize(){
     if ( this->nLoci_ != this->altCount_.size() ){
         throw LociNumberUnequal( this->altFileName_ );
     }
-
     (void)removeFilesWithSameName();
+
+    this->readPanel();
+    IBDpathChangeAt = vector <double>(this->nLoci());
+    finalIBDpathChangeAt = vector <double>(this->nLoci());
+
+    siteOfTwoSwitchOne = vector <double>(this->nLoci());
+    siteOfTwoMissCopyOne = vector <double>(this->nLoci());
+    siteOfTwoSwitchTwo = vector <double>(this->nLoci());
+    siteOfTwoMissCopyTwo = vector <double>(this->nLoci());
+    siteOfOneSwitchOne = vector <double>(this->nLoci());
+    siteOfOneMissCopyOne = vector <double>(this->nLoci());
+
+    finalSiteOfTwoSwitchOne = vector <double>(this->nLoci());
+    finalSiteOfTwoMissCopyOne = vector <double>(this->nLoci());
+    finalSiteOfTwoSwitchTwo = vector <double>(this->nLoci());
+    finalSiteOfTwoMissCopyTwo = vector <double>(this->nLoci());
+    finalSiteOfOneSwitchOne = vector <double>(this->nLoci());
+    finalSiteOfOneMissCopyOne = vector <double>(this->nLoci());
 }
 
 
 void DEploidIO::removeFilesWithSameName(){
+    strExportProp = this->prefix_ + ".prop";
     strExportLLK = this->prefix_ + ".llk";
     strExportHap = this->prefix_ + ".hap";
 
+    strIbdExportProp = this->prefix_ + ".ibd.prop";
+    strIbdExportLLK = this->prefix_ + ".ibd.llk";
+    strIbdExportHap = this->prefix_ + ".ibd.hap";
+
     strExportVcf = this->prefix_ + ".vcf";
     if ( compressVcf() ){
         strExportVcf += ".gz";
     }
-    strExportProp = this->prefix_ + ".prop";
-    strExportLog =  this->prefix_ + ".log";
+    strExportLog =  this->prefix_ + ((this->doPainting()) ? ".painting":"") + ".log";
     strExportRecombProb = this->prefix_ + ".recomb";
 
-    strExportOneSwitchOne   = this->prefix_ + ".oneSwitchOne";
-    strExportOneMissCopyOne = this->prefix_ + ".oneMissCopyOne";
-    strExportTwoSwitchOne   = this->prefix_ + ".twoSwitchOne";
-    strExportTwoMissCopyOne = this->prefix_ + ".twoMissCopyOne";
-    strExportTwoSwitchTwo   = this->prefix_ + ".twoSwitchTwo";
-    strExportTwoMissCopyTwo = this->prefix_ + ".twoMissCopyTwo";
+    strExportExtra = this->prefix_ + ".extra";
 
-
-    remove(strExportLLK.c_str());
-    remove(strExportHap.c_str());
-    remove(strExportVcf.c_str());
-    remove(strExportProp.c_str());
-    remove(strExportLog.c_str());
-    remove(strExportRecombProb.c_str());
-
-    remove(strExportOneSwitchOne.c_str());
-    remove(strExportOneMissCopyOne.c_str());
-    remove(strExportTwoSwitchOne.c_str());
-    remove(strExportTwoMissCopyOne.c_str());
-    remove(strExportTwoSwitchTwo.c_str());
-    remove(strExportTwoMissCopyTwo.c_str());
-
-    strExportSingleFwdProbPrefix = this->prefix_ + ".single";
-    for ( size_t i = 0; i < this->kStrain_ ; i++ ){
-        string tmpStrExportSingleFwdProb = strExportSingleFwdProbPrefix + to_string(i);
-        remove(tmpStrExportSingleFwdProb.c_str());
+    if ( this->doPainting() == false ){
+        if (this->useIBD()){
+            remove(strIbdExportProp.c_str());
+            remove(strIbdExportLLK.c_str());
+            remove(strIbdExportHap.c_str());
+        }
+        remove(strExportLLK.c_str());
+        remove(strExportHap.c_str());
+        remove(strExportVcf.c_str());
+        remove(strExportProp.c_str());
+        remove(strExportExtra.c_str());
     }
 
-    strExportPairFwdProb = this->prefix_ + ".pair";
-    remove(strExportPairFwdProb.c_str());
+    if (this->doPainting() || this->doExportPostProb() ){
+        if (this->useIBD()){
+            strIbdExportSingleFwdProbPrefix = this->prefix_ + ".ibd.single";
+            for ( size_t i = 0; i < this->kStrain_ ; i++ ){
+                string tmpStrExportSingleFwdProb = strIbdExportSingleFwdProbPrefix + to_string(i);
+                remove(tmpStrExportSingleFwdProb.c_str());
+            }
+            strIbdExportPairFwdProb = this->prefix_ + ".ibd.pair";
+            remove(strIbdExportPairFwdProb.c_str());
+        }
+        strExportSingleFwdProbPrefix = this->prefix_ + ".single";
+        for ( size_t i = 0; i < this->kStrain_ ; i++ ){
+            string tmpStrExportSingleFwdProb = strExportSingleFwdProbPrefix + to_string(i);
+            remove(tmpStrExportSingleFwdProb.c_str());
+        }
+        strExportPairFwdProb = this->prefix_ + ".pair";
+        remove(strExportPairFwdProb.c_str());
+    }
+    remove(strExportLog.c_str());
+    remove(strExportRecombProb.c_str());
 
 }
 
@@ -295,7 +341,10 @@ void DEploidIO::parse (){
             if ( doExportPostProb() ){
                 throw ( FlagsConflict((*argv_i) , "-exportPostProb") );
             }
-            this->set_panel(false);
+            if ( doAllowInbreeding() ){
+                throw ( FlagsConflict((*argv_i) , "-inbreeding") );
+            }
+            this->setUsePanel(false);
             this->setDoExportSwitchMissCopy ( false );
         } else if (*argv_i == "-exclude"){
             this->setExcludeSites( true );
@@ -305,7 +354,19 @@ void DEploidIO::parse (){
         } else if ( *argv_i == "-p" ) {
             this->precision_ = readNextInput<size_t>() ;
         } else if ( *argv_i == "-k" ) {
-            this->kStrain_ = readNextInput<size_t>() ;
+            this->setKStrainWasManuallySet(true);
+            this->setKstrain(readNextInput<size_t>());
+
+            if ( this->kStrainWasSetByHap() && this->kStrain() != this->initialHap[0].size() ){
+                string hint = string(" k = ") + to_string(this->kStrain()) + ", " + this->initialHapFileName_ + " suggests otherwise";
+                throw NumOfPropNotMatchNumStrain(hint);
+            }
+
+            if ( this->initialPropWasGiven() && this->kStrain_ != initialProp.size() ){
+                string hint = string(" k = ") + to_string(kStrain_) + ", flag -initialP suggests otherwise";;
+                throw NumOfPropNotMatchNumStrain(hint);
+            }
+
         } else if ( *argv_i == "-nSample" ) {
             this->nMcmcSample_ = readNextInput<size_t>() ;
         } else if ( *argv_i == "-burn" ) {
@@ -318,6 +379,12 @@ void DEploidIO::parse (){
             if ( this->missCopyProb_ < 0 || this->missCopyProb_ > 1){
                 throw ( OutOfRange ("-miss", *argv_i) );
             }
+        } else if ( *argv_i == "-c" ) {
+            this->scalingFactor_ = readNextInput<double>() ;
+        } else if ( *argv_i == "-G" ) {
+            this->setParameterG(readNextInput<double>());
+        } else if ( *argv_i == "-sigma" ) {
+            this->setParameterSigma(readNextInput<double>());
         } else if ( *argv_i == "-recomb" ) {
             this->constRecombProb_ = readNextInput<double>();
             this->useConstRecomb_ = true;
@@ -336,14 +403,53 @@ void DEploidIO::parse (){
             this->setDoUpdateSingle( false );
         } else if ( *argv_i == "-forbidUpdatePair" ) {
             this->setDoUpdatePair( false );
+        } else if ( *argv_i == "-inbreeding" ) {
+            if ( this->usePanel() == false ){
+                throw ( FlagsConflict((*argv_i) , "-noPanel") );
+            }
+            this->setDoAllowInbreeding( true );
+            this->setDoExportPostProb( true );
         } else if ( *argv_i == "-exportPostProb" ) {
             if ( this->usePanel() == false ){
                 throw ( FlagsConflict((*argv_i) , "-noPanel") );
             }
             this->setDoExportPostProb( true );
+        } else if ( *argv_i == "-painting" ) {
+            if ( this->usePanel() == false ){
+                throw ( FlagsConflict((*argv_i) , "-noPanel") );
+            }
+            if ( this->initialHapWasGiven() == true ){
+                throw ( FlagsConflict((*argv_i) , "-initialHap") );
+            }
+            this->readNextStringto ( this->initialHapFileName_ ) ;
+            this->setDoPainting( true );
+            this->readInitialHaps();
+        } else if ( *argv_i == "-ibd" ){
+            this->setUseIBD(true);
         } else if ( *argv_i == "-initialP" ){
             this->readInitialProportions();
-            this->initialPropWasGiven_ = true;
+            this->setInitialPropWasGiven( true );
+
+            // If the k was set manually, check
+            if ( this->kStrainWasManuallySet() && this->kStrain_ != initialProp.size() ){
+                string hint = string(" k = ") + to_string(kStrain_);
+                throw NumOfPropNotMatchNumStrain(hint);
+            }
+
+            // If the k was set by initial Hap, check
+            if ( this->kStrainWasSetByHap() && this->kStrain() != this->initialProp.size() ){
+                string hint = string(" k = ") + to_string(this->kStrain()) + ", " + this->initialHapFileName_ + " suggests otherwise";
+                throw NumOfPropNotMatchNumStrain(hint);
+            }
+
+            this->setKstrain(this->initialProp.size());
+        } else if ( *argv_i == "-initialHap" ){
+            if ( this->doPainting() == true ){
+                throw ( FlagsConflict((*argv_i) , "-painting") );
+            }
+            this->readNextStringto ( this->initialHapFileName_ ) ;
+            this->setInitialHapWasGiven(true);
+            this->readInitialHaps();
         } else if ( *argv_i == "-seed"){
             this->set_seed( readNextInput<size_t>() );
             this->setRandomSeedWasSet( true );
@@ -370,10 +476,14 @@ void DEploidIO::checkInput(){
         throw FileNameMissing ( "PLAF" );}
     if ( usePanel() && this->panelFileName_.size() == 0 ){
         throw FileNameMissing ( "Reference panel" );}
-    if ( this->initialPropWasGiven() && ( abs(sumOfVec(initialProp) - 1.0) > 0.000001 )){
+    if ( this->initialPropWasGiven() && ( abs(sumOfVec(initialProp) - 1.0) > 0.00001 )){
         throw SumOfPropNotOne ( to_string(sumOfVec(initialProp)) );}
-    if ( this->initialPropWasGiven() && kStrain_ != initialProp.size() ){
-        throw NumOfPropNotMatchNumStrain(""); }
+    if ( this->initialPropWasGiven() ){
+        if ( this->kStrainWasManuallySet() == true ){
+        } else {
+            // set k strain by proportion length
+        }
+    }
 }
 
 
@@ -426,6 +536,7 @@ void DEploidIO::printHelp(std::ostream& out){
         << endl;
     out << setw(20) << "-h or -help"         << "  --  " << "Help. List the following content."<<endl;
     out << setw(20) << "-v or -version"      << "  --  " << "DEploid version."<<endl;
+    out << setw(20) << "-vcf STR"            << "  --  " << "VCF file path."<<endl;
     out << setw(20) << "-ref STR"            << "  --  " << "File path of reference allele count."<<endl;
     out << setw(20) << "-alt STR"            << "  --  " << "File path of alternative allele count."<<endl;
     out << setw(20) << "-plaf STR"           << "  --  " << "File path of population level allele frequencies."<<endl;
@@ -447,6 +558,9 @@ void DEploidIO::printHelp(std::ostream& out){
     out << endl;
     out << "./dEploid -vcf data/testData/PG0390-C.test.vcf -plaf data/testData/labStrains.test.PLAF.txt -o PG0390-CNopanel -noPanel"<< endl;
     out << "./dEploid -vcf data/testData/PG0390-C.test.vcf -exclude data/testData/labStrains.test.exclude.txt -plaf data/testData/labStrains.test.PLAF.txt -o PG0390-CNopanelExclude -noPanel"<< endl;
+    out << "./dEploid -vcf data/testData/PG0390-C.test.vcf -exclude data/testData/labStrains.test.exclude.txt -plaf data/testData/labStrains.test.PLAF.txt -o PG0390-CPanelExclude -panel data/testData/labStrains.test.panel.txt" << endl;
+    out << "./dEploid -vcf data/testData/PG0390-C.test.vcf -exclude data/testData/labStrains.test.exclude.txt -plaf data/testData/labStrains.test.PLAF.txt -o PG0390-CPanelExclude -panel data/testData/labStrains.test.panel.txt -painting PG0390-CPanelExclude.hap" << endl;
+    out << "./dEploid -vcf data/testData/PG0390-C.test.vcf -plaf data/testData/labStrains.test.PLAF.txt -o PG0390-CNopanel -noPanel -k 2 -ibd -nSample 250 -rate 8 -burn 0.67" <<endl;
 }
 
 
@@ -454,3 +568,122 @@ std::ostream& operator<< (std::ostream& stream, const DEploidIO& dEploidIO) {
   for (std::string arg : dEploidIO.argv_) stream << " " << arg;
   return stream;
 }
+
+
+void DEploidIO::readInitialHaps(){
+    assert( this->initialHap.size() == 0 );
+    InitialHaplotypes initialHapToBeRead;
+    initialHapToBeRead.readFromFile(this->initialHapFileName_.c_str());
+
+    assert (this->initialHap.size() == 0 );
+    this->initialHap = initialHapToBeRead.content_;
+
+    if ( this->kStrainWasManuallySet() && this->kStrain()!= initialHapToBeRead.truePanelSize() ){
+        string hint = string(" k = ") + to_string(this->kStrain()) + ", " + this->initialHapFileName_ + " suggests otherwise";
+        throw NumOfPropNotMatchNumStrain(hint);
+    }
+
+    if ( this->kStrainWasSetByProp() && this->kStrain() != initialHapToBeRead.truePanelSize() ){
+        string hint = string(" k = ") + to_string(kStrain_) + ", flag -initialP suggests otherwise";;
+        throw NumOfPropNotMatchNumStrain(hint);
+    }
+
+    this->setKstrain(initialHapToBeRead.truePanelSize());
+    this->setKStrainWasSetByHap(true);
+}
+
+
+void DEploidIO::chromPainting(){
+    dout << "Painting haplotypes in" << this->initialHapFileName_ <<endl;
+
+    if ( this->initialPropWasGiven() == false ){
+        clog << "Initial proportion was not specified. Set even proportions" << endl;
+        double evenProp = 1.0 / (double)this->kStrain();
+        for ( size_t i = 0; i < this->kStrain(); i++){
+            this->initialProp.push_back(evenProp);
+        }
+    }
+
+    for ( auto const& value: this->initialProp ){
+        this->filnalProp.push_back(value);
+    }
+
+    // Painting posterior probabilities
+
+    // Export the p'
+    // Make this a separate class
+    //vector < vector <double> > hap;
+    //for ( size_t siteI = 0; siteI < decovolutedStrainsToBeRead.content_.size(); siteI++ ){
+        //vector <double> tmpHap;
+        //for ( size_t tmpk = 0; tmpk < this->kStrain_; tmpk++ ){
+            //tmpHap.push_back(decovolutedStrainsToBeRead.content_[siteI][tmpk]);
+        //}
+        //hap.push_back(tmpHap);
+    //}
+
+    //vector < vector <double>> hap = decovolutedStrainsToBeRead.content_;
+
+    // Make this a separate function
+    // calculate expected wsaf
+    vector <double> expectedWsaf (this->nLoci_, 0.0);
+    for ( size_t i = 0; i < this->initialHap.size(); i++ ){
+        assert( kStrain_ == this->initialHap[i].size() );
+        for ( size_t k = 0; k < this->kStrain_; k++){
+            expectedWsaf[i] += this->initialHap[i][k] * filnalProp[k];
+        }
+        assert ( expectedWsaf[i] >= 0 );
+        //assert ( expectedWsaf[i] <= 1.0 );
+    }
+
+    MersenneTwister tmpRg(this->randomSeed());
+
+    if ( this->doAllowInbreeding() == true ){
+        this->panel->initializeUpdatePanel(this->panel->truePanelSize()+kStrain_-1);
+    }
+
+    for ( size_t tmpk = 0; tmpk < this->kStrain_; tmpk++ ){
+        if ( this->doAllowInbreeding() == true ){
+            this->panel->updatePanelWithHaps(this->panel->truePanelSize()+kStrain_-1, tmpk, this->initialHap);
+        }
+
+        for ( size_t chromi = 0 ; chromi < this->indexOfChromStarts_.size(); chromi++ ){
+            size_t start = this->indexOfChromStarts_[chromi];
+            size_t length = this->position_[chromi].size();
+            dout << "Painting Chrom "<< chromi << " from site "<< start << " to " << start+length << endl;
+
+            UpdateSingleHap updatingSingle( this->refCount_,
+                                      this->altCount_,
+                                      this->plaf_,
+                                      expectedWsaf,
+                                      this->filnalProp, this->initialHap, &tmpRg,
+                                      start, length,
+                                      this->panel, this->missCopyProb_, this->scalingFactor(),
+                                      tmpk);
+
+            if ( this->doAllowInbreeding() == true ){
+                updatingSingle.setPanelSize(this->panel->inbreedingPanelSize());
+            }
+            updatingSingle.painting( refCount_, altCount_, expectedWsaf, this->filnalProp, this->initialHap);
+            //this->writeLastSingleFwdProb( updatingSingle.fwdProbs_, chromi, tmpk, false ); // false as not using ibd
+            this->writeLastSingleFwdProb( updatingSingle.fwdBwdProbs_, chromi, tmpk, false ); // false as not using ibd
+        }
+    }
+}
+
+
+void DEploidIO::readPanel(){
+    if ( this->usePanel() == false ){
+        return;
+    }
+
+    panel = new Panel();
+    panel->readFromFile(this->panelFileName_.c_str());
+    if ( this->excludeSites() ){
+        panel->findAndKeepMarkers( this->excludedMarkers );
+    }
+
+    panel->computeRecombProbs( this->averageCentimorganDistance(), this->parameterG(), this->useConstRecomb(), this->constRecombProb(), this->forbidCopyFromSame() );
+    panel->checkForExceptions( this->nLoci(), this->panelFileName_ );
+}
+
+
